import { AuditLogEntryCard } from "@/components/cards/AuditLogEntryCard";
import { CollapsibleSection } from "@/components/common/CollapsibleSection";
import { Permission } from "@/lib/accessPermissions";
import { fetchAuditLogEvents } from "@/lib/actions";
import { AuditLogEntry } from "@/lib/db";

const collapseGroups: { perms: Permission[], name: string }[] = [
  {
    name: "user",
    perms: [
      "users/fetch/by-id",
      "users/fetch/memberships",
      "users/fetch/notices",
      "users/fetch/relations",
      "users/fetch/strikes",
      "reports/fetch/related/by-user",
      "reports/fetch/related/against-user",
    ],
  },
  {
     name: "account",
     perms: [
      "accounts/fetch/by-id",
      "accounts/fetch/by-email",
      "sessions/fetch/by-account-id",
    ],
  },
  {
    name: "server",
    perms: [
      "servers/fetch/by-id",
    ],
  },
  {
    name: "channel",
    perms: [
      "channels/fetch/by-id",
      "channels/fetch/by-server",
    ]
  },
  {
    name: "report",
    perms: [
      "reports/fetch/by-id",
      "reports/fetch/snapshots/by-report",
    ],
  }
];

const getGroup = (permission: string) => collapseGroups.findIndex((list) => list.perms.find((p) => p.startsWith(permission)));

export default async function AuditLog() {
  const entries = await fetchAuditLogEvents();
  const items: (AuditLogEntry | AuditLogEntry[])[] = [];
  let tmp: AuditLogEntry[] = [];

  for (const entry of entries) {
    const group = getGroup(entry.permission);

    if (tmp.length && group == getGroup(tmp[0].permission)) {
      tmp.push(entry);
    } else {
      if (tmp.length) {
        items.push(tmp);
        tmp = [];
      }

      if (group != -1) tmp.push(entry);
      else items.push(entry);
    }
  }
  items.push(...tmp);

  return (
    <div className="flex flex-col gap-2">
      {items.map((entry) => Array.isArray(entry)
        ? (
          <CollapsibleSection
            text={`${entry.length} ${collapseGroups[getGroup(entry[0].permission)].name} fetch event${entry.length == 1 ? "" : "s"}`}
            key={entries.indexOf(entry as any)}
          >
            <div className="flex flex-col gap-2">
              {entry.map((item) => <AuditLogEntryCard key={item._id} log={item} />)}
            </div>
          </CollapsibleSection>
        )
        : (
          <AuditLogEntryCard key={entry._id} log={entry} />
        )
      )}
    </div>
  );
}
